import os
from pykindle import readers


class FieldNotSetError(Exception):
    """
    When user get a field of an item that has not been set yet, raise.
    """

    def __init__(self, field):
        super(FieldNotSetError, self).__init__('{} not set.'.format(field))


class Item(dict):
    """
    Item represents an asset of mobi files.
    This is a base class, you should inherit from it.

    Mobi file consists of several assets and a manifest.
    Each item renders to a asset,
    including articles, images and manifest files.

    Item class inherited from python dict class.
    You can set attributes in two ways:

    >>> item = Item()
    >>> item['href'] = 'path/to/item'
    >>> assert item.href == 'path/to/item'
    >>> item.href = 'path/to/item/changed'
    >>> assert item['href'] = 'path/to/item/changed'

    """

    def __init__(self, *args, **kwargs):
        super(Item, self).__init__(*args, **kwargs)
        self._default = dict()

    def __getitem__(self, key):
        assert isinstance(key, str)
        if key in self:
            return super(Item, self).__getitem__(key)
        elif key in self._default:
            return self._default[key]
        else:
            raise FieldNotSetError(key)

    def __setitem__(self, key, value):
        super(Item, self).__setitem__(key, value)

    @property
    def href(self):
        """
        Mobi file is just like a html website,
        and each asset has it's relative url in the mobi file.
        Html assets get other html assets and image assets by relative url.
        """
        return self['href']

    @href.setter
    def href(self, value):
        assert isinstance(value, str)
        self['href'] = value

    @property
    def media_type(self):
        """
        Every mobi asset has a field to indicates what it is, for example,
        application/xhtml+xml represents an article,
        image/jpg represents a photo in jpg format.
        """
        return self['media_type']

    @media_type.setter
    def media_type(self, value):
        assert isinstance(value, str)
        assert '/' in value
        self['media_type'] = value

    @property
    def id(self):
        """
        Id field should be a string! Not integer!
        Just like html elements has a "id" field,
        mobi assets also have such field.
        The difference is that in mobi assets, "id" field in required.
        """
        return self['id']

    @id.setter
    def id(self, value):
        assert isinstance(value, str)
        self['id'] = value

    @property
    def source(self):
        """
        Source is the main content of the asset.
        It will later send to self.reader.render to get the rendered result.
        """
        return self['source']

    @source.setter
    def source(self, value):
        assert value is not None
        self['source'] = value

    @property
    def reader(self):
        """
        Smetimes we need to convert some files to mobi supported assets.
        That's why we use reader.
        Reader should be a instance of pykindle.reader.Reader class.
        Item will call self.reader.render(self.source) to generate rendered assets.
        """
        return self['reader']

    @reader.setter
    def reader(self, value):
        assert isinstance(value, readers.Reader)
        self['reader'] = value

    @property
    def content(self):
        """
        Readonly field.
        It just return the value of self.reader.render(self.source).
        """
        return self.reader.render(self.source)

    def write(self, directory):
        """
        Write the rendered content to a file.
        File path is calculated by os.path.join(directory, self.href).
        """
        path = os.path.join(directory, *self.href.split('/'))
        with open(path, mode='w', encoding='utf-8') as file:
            file.write(self.content)


class ArticleItem(Item):
    """
    ArticleItem represents an real html article.
    This is the main body of your book.

    This class is used for inherit.

    Default attributes:
        title = Untitled
        author = No Author
        description = No description
        category = default
    """

    def __init__(self, *args, **kwargs):
        super(ArticleItem, self).__init__(*args, **kwargs)
        self.media_type = 'application/xhtml+xml'
        self._default.update({
            'title': 'Untitled',
            'author': 'No Author',
            'description': 'No description.',
            'category': 'default',
        })

    @property
    def title(self):
        """
        The title of article.
        """
        return self['title']

    @title.setter
    def title(self, value):
        assert isinstance(value, str)
        self['title'] = value

    @property
    def author(self):
        """
        The author of article.
        """
        return self['author']

    @author.setter
    def author(self, value):
        assert isinstance(value, str)
        self['author'] = value

    @property
    def description(self):
        """
        The description of article.
        """
        return self['description']

    @description.setter
    def description(self, value):
        assert isinstance(value, str)
        self['description'] = value

    @property
    def category(self):
        """
        Optional.
        If you generates a magazine book, this field is used to calculate the magazine category.
        Deprecated. We will create a Category class to do this.
        :return:
        """
        return self['category']

    @category.setter
    def category(self, value):
        assert isinstance(value, str)
        self['category'] = value

    @property
    def content(self):
        return self.reader.render(self.title, self.author, self.description, self.source)


class HtmlArticleItem(ArticleItem):
    """
    HtmlArticleItem represents a html article.
    Although all the articles in mobi files are html,
    we have other ArticleItem like MarkdownArticleItem to convert other formats to html.
    """

    def __init__(self, *args, **kwargs):
        super(HtmlArticleItem, self).__init__(*args, **kwargs)
        self.reader = readers.HtmlReader()


class MarkdownArticleItem(ArticleItem):
    """
    MarkdownArticleItem reads a markdown source.
    """

    def __init__(self, *args, **kwargs):
        super(MarkdownArticleItem, self).__init__(*args, **kwargs)
        self.reader = readers.MarkdownReader()


class ImageItem(Item):
    """
    ImageItem represents a image asset.
    It has a positional argument, image_format='jpeg', alternative formats are jpeg, png, gif
    """
    def __init__(self, image_format='jpeg', *args, **kwargs):
        super(ImageItem, self).__init__(*args, **kwargs)
        assert image_format in ('jpeg', 'png', 'gif')
        self.image_format = image_format
        self.media_type = 'image/' + image_format
        self.reader = readers.ImageReader()


class CoverImageItem(ImageItem):
    """
    CoverImageItem represents a cover image.
    """
    pass


class NcxItem(Item):
    """
    NcxItem is used to generate ncx file.
    Ncx file is the category of a book.
    It will be automatically added to the book when book.create is called.
    """
    def __init__(self, *args, **kwargs):
        super(NcxItem, self).__init__(*args, **kwargs)
        self.media_type = 'application/x-dtbncx+xml'
        self._default.update({
            'id': 'ncx',
            'href': 'ncx.toc',
            'title': 'Untitled',
            'author': 'No Author',
        })

    @property
    def title(self):
        return self['title']

    @title.setter
    def title(self, value):
        assert isinstance(value, str)
        self['title'] = value

    @property
    def author(self):
        return self['author']

    @author.setter
    def author(self, value):
        assert isinstance(value, str)
        self['author'] = value


class MagazineNcxItem(NcxItem):
    """
    MagazineNcxItem represents a magazine toc.
    The way to implement magazine is very ugly, and we'll reconstruct these classes.
    """
    def __init__(self, *args, **kwargs):
        super(MagazineNcxItem, self).__init__(*args, **kwargs)
        self.reader = readers.MagazineNcxReader()

    @property
    def categories(self):
        return self['categories']

    @categories.setter
    def categories(self, value):
        assert isinstance(value, list)
        for category, articles in value:
            if not isinstance(category, str):
                raise ValueError('category should be a string.')
            for article in articles:
                if not isinstance(article, ArticleItem):
                    raise ValueError('article should be a pykindle.items.Article object')
        self['categories'] = value

    @property
    def content(self):
        return self.reader.render(title=self.title, author=self.author, categories=self.categories)


__all__ = ['Item', 'ArticleItem', 'HtmlArticleItem', 'MarkdownArticleItem', 'ImageItem', 'CoverImageItem', 'NcxItem',
           'MagazineNcxItem', 'FieldNotSetError']



from pykindle import readers
from pykindle import items


def test_html_render():
    reader = readers.HtmlReader()
    assert reader.render(title='title', author='author', description='description', content='content')


def test_markdown_render():
    reader = readers.MarkdownReader()
    assert reader.render(title='title', author='author', description='description', content='content')


def test_magazine_ncx_render():
    reader = readers.MagazineNcxReader()
    try:
        reader.render(categories=[], title='title', author='author')
        raise AssertionError
    except ValueError:
        pass
    articles = [items.ArticleItem() for i in range(10)]
    for index, article in enumerate(articles):
        article.href = '{}.html'.format(index)
        article.id = str(index)
    assert reader.render(categories=[
        ('category1', articles[:5]),
        ('category2', articles[5:]),
    ], title='title', author='author')



import os
import datetime
import tempfile
import jinja2
from pykindle import items as _items
from pykindle import readers as _readers


class Book(list):
    """
    This is a base class.
    You should extend it, rather than use it directly.
    See MagazineBook.

    This class is inherited from python list class.
    You can use all list functions.

    Required attributes: None

    Alternative attributes: title, language, uid, creator, publisher, subject, author, date, description
    """

    def __init__(self):
        super(Book, self).__init__()
        self.title = 'Auto Generted by PyKindle'
        self.language = 'zh-cn'
        self.uid = '0000000000'
        self.creator = 'PyKindle'
        self.publisher = 'https://github.com/panhaoyu/pykindle'
        self.subject = 'PyKindle'
        self.author = 'Yourself'
        self.date = datetime.datetime.now().strftime('%Y-%m-%d')
        self.description = 'Auto generated by pykindle. You can get more information at our website: https://github.com/panhaoyu/pykindle.'
        self._ncx = None

        self._jinja = jinja2.Environment(
            loader=jinja2.FileSystemLoader(os.path.join(os.path.dirname(os.path.relpath(__file__)), 'templates')))

    def _write_items(self, directory):
        for item in self:
            item.write(directory)
        self._ncx.write(directory)

    def _write_opf(self, directory):
        content = self._jinja.get_template('magazine.opf').render({
            'title': self.title, 'language': self.language, 'uid': self.uid, 'creator': self.creator,
            'publisher': self.publisher, 'subject': self.subject, 'date': self.date, 'description': self.description,
            'cover_image_url': self._cover_image_url, 'ncx': self._ncx, 'items': self, 'articles': self._articles
        })
        with open(os.path.join(directory, 'index.opf'), mode='w', encoding='utf-8') as file:
            file.write(content)

    def _generate_ncx(self):
        self._ncx = _items.MagazineNcxItem()
        self._ncx.id = 'ncx'
        self._ncx.path = 'toc.ncx'
        self._ncx.title = self.title
        self._ncx.author = self.author
        categories = dict()
        for item in self:
            category = categories.setdefault(item.category, list())
            category.append(item)
        self._ncx.categories = list(categories.items())

    def _generate_book(self, directory):
        opf_file = os.path.join(directory, 'index.opf')
        os.system('kindlegen {} -o output.mobi'.format(opf_file))
        output_path = os.path.join(directory, 'output.mobi')
        if not os.path.exists(output_path):
            raise ValueError('Mobi file generated failed. Please raise an issue.')
        with open(output_path, mode='rb') as file:
            content = file.read()
        return content

    @property
    def _cover_image_url(self):
        for item in self:
            if isinstance(item, _items.CoverImageItem):
                return item.href

    @property
    def _articles(self):
        return [item for item in self if isinstance(item, _items.ArticleItem)]

    def _generate_article_id(self):
        for index, item in enumerate(self):
            item.id = str(index)

    def create(self):
        with tempfile.TemporaryDirectory() as directory:
            self._generate_article_id()
            self._generate_ncx()
            self._write_opf(directory)
            self._write_items(directory)
            content = self._generate_book(directory)
        return content


class NormalBook(Book):
    """
    This class represents a normal book.
    Normal means the book has a toc, and readers will read from first paragraph to last paragraph.
    """
    pass


class MagazineBook(Book):
    pass


__all__ = ['Book', 'MagazineBook', 'NormalBook']



from pykindle import items


def test_get_field():
    item = items.Item()
    item.id = "test_id"
    assert isinstance(item.id, str)


def test_get_empty_field():
    item = items.Item()
    try:
        i = item.id
        raise AssertionError
    except items.FieldNotSetError:
        pass


def test_get_default_field():
    item = items.MarkdownArticleItem()
    assert isinstance(item.title, str)


def test_get_non_exist_field():
    item = items.NcxItem()
    try:
        nothing = item.it_is_not_an_item
        assert 0
    except AttributeError:
        pass


def test_set_field_twice():
    item = items.Item()
    item.href = 'path/to/item'
    item.href = 'path/to/item'
    assert item.href == 'path/to/item'


def test_get_field_twice():
    item = items.Item()
    item.href = 'path/to/item'
    assert item.href == 'path/to/item'
    assert item.href == 'path/to/item'


def test_set_readonly_field():
    item = items.Item()
    try:
        item.content = 'nothing'
        assert 0
    except AttributeError:
        pass


def test_item():
    item = items.Item()
    item.href = 'path/to/item'
    assert item.href == 'path/to/item'
    item.media_type = 'application/xhtml+xml'
    assert item.media_type == 'application/xhtml+xml'
    item.id = 'temp_id'
    assert item.id == 'temp_id'
    item.source = 'This is a source line.'
    assert item.source == 'This is a source line.'


def test_article():
    item = items.ArticleItem()
    assert isinstance(item.title, str)
    assert isinstance(item.author, str)
    assert isinstance(item.description, str)
    assert isinstance(item.category, str)
    item.title = 'this is a title'
    assert item.title == 'this is a title'
    item.author = 'this is a author'
    assert item.author == 'this is a author'
    item.description = 'this is a description'
    assert item.description == 'this is a description'
    item.category = 'category'
    assert item.category == 'category'


def test_ncx_item():
    item = items.NcxItem()
    assert isinstance(item.title, str)
    assert isinstance(item.author, str)
    assert isinstance(item.id, str)
    assert isinstance(item.href, str)
    item.id = 'id'
    assert item.id == 'id'
    item.href = 'href'
    assert item.href == 'href'
    item.title = 'ttt'
    assert item.title == 'ttt'
    item.author = 'aaa'
    assert item.author == 'aaa'


def test_magazine_ncx_item():
    item = items.MagazineNcxItem()
    item.categories = list()
    item.categories = [('test_category', [
        items.ArticleItem(), items.ArticleItem()
    ]), ]
    assert item.categories[0][0] == 'test_category'
    item.categories = [
        ('test_category', [items.ArticleItem(), items.ArticleItem()]),
        ('test_category2', [items.ArticleItem(), items.ArticleItem()]),
        ('test_category3', [items.ArticleItem(), items.ArticleItem()]),
    ]
    assert item.categories[2][0] == 'test_category3'


def test_image_item():
    assert True


def test_reader():
    assert True


def test_write():
    assert True



import os
import markdown
import jinja2

_template_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates')


class Reader(object):
    """
    This class is only for inherit.
    Reader is in fact a package of the render function of jinja.
    It renders some args to a mobi asset.
    Renders are only used in items, items will call item.reader.render when generate a book.
    """

    def render(self, *args, **kwargs):
        """
        Render some source to mobi asset.
        """
        pass


class TemplateReader(Reader):
    """
    This class is only used for inherit.
    It defines the jinja2 environment.
    """

    def __init__(self, jinja2_loader=jinja2.FileSystemLoader(_template_dir)):
        self._jinja2 = jinja2.Environment(loader=jinja2_loader)


class HtmlReader(TemplateReader):
    """
    This class renders html source code.
    """

    def render(self, title, author, description, content):
        """
        Title, author, description and content, as their names indicate, are the metadata of the html article.
        """
        super(HtmlReader, self).render()
        template = self._jinja2.get_template('article.html')
        return template.render({
            'title': title,
            'author': author,
            'description': description,
            'content': content,
        })


class MarkdownReader(HtmlReader):
    """
    This class renders markdown source code to mobi asset.
    """

    def __init__(self, markdown_options=None, **kwargs):
        """
        TODO: write a testcase for markdown_options
        """
        super(MarkdownReader, self).__init__(**kwargs)
        self._markdown_options = markdown_options if markdown_options else dict()

    def render(self, title, author, description, content):
        """
        Title, author, description and content, as their names indicate, are the metadata of the html article.
        """
        content = markdown.markdown(content, **self._markdown_options)
        return super(MarkdownReader, self).render(title, author, description, content)


class ImageReader(Reader):
    """
    Used to render image.
    Currently, it just return the source itself.
    """
    def render(self, content):
        return content


class NcxReader(TemplateReader):
    """
    TODO: implement.
    """
    pass


class MagazineNcxReader(NcxReader):
    """
    Render the ncx asset.
    """
    def render(self, categories, title, author):
        """
        It's not stable currently.
        TODO: change the categories behavior.
        """
        assert isinstance(categories, list)
        if len(categories) == 0:
            raise ValueError('There is no category.')
        for category, articles in categories:
            if len(articles) == 0:
                raise ValueError('There is no article in {}'.format(category))
        template = self._jinja2.get_template('magazine.ncx')
        result = template.render({
            'categories': categories,
            'title': title,
            'author': author
        })
        return result


__all__ = ['Reader', 'TemplateReader', 'HtmlReader', 'MarkdownReader', 'MagazineNcxReader', 'NcxReader', 'ImageReader']



import os
import tempfile
import pykindle


# I get the test data from https://github.com/jachinlin/kindle_maker/tree/master/examples/source.

def test_magazine():
    book = pykindle.MagazineBook()
    base_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data')
    contents = {
        '技术领导力300讲': [
            '开篇词：卓越的团队，必然有一个卓越的领导者',
            '第1讲：你的能力模型决定你的职位',
            '大咖对话：从几个工程师到2000more个工程师的技术团队成长秘诀',
            '第14讲：从零开始搭建轻量级研发团队',
        ],
        '硅谷产品实战36讲': [
            '开篇词：打造千万用户的世界级产品',
            '01：什么是优秀的产品经理？',
            '02：硅谷的产品经理是什么样子的？',
        ]
    }
    for category, articles in contents.items():
        for article_file_name in articles:
            path = os.path.join(base_path, '{}.html'.format(article_file_name))
            with open(path, mode='r', encoding='utf-8') as file:
                content = file.read()
            article = pykindle.HtmlArticleItem()
            article.category = category
            article.title = article_file_name
            article.source = content
            article.href = os.path.split(path)[1]
            book.append(article)
    content = book.create()
    with tempfile.TemporaryDirectory() as directory:
        with open(os.path.join(directory, 'output.mobi'), mode='wb') as file:
            file.write(content)



from pykindle.book import NormalBook, MagazineBook
from pykindle.items import ArticleItem, ImageItem, CoverImageItem, HtmlArticleItem, MarkdownArticleItem

__all__ = [
    'NormalBook', 'MagazineBook',
    'ArticleItem', 'HtmlArticleItem', 'MarkdownArticleItem', 'ImageItem', 'CoverImageItem'
]



#!/usr/bin/env python
import configparser
from setuptools import find_packages, setup

config = configparser.ConfigParser()
with open('config.ini', mode='r', encoding='utf-8') as file:
    config.read_file(file)


def get_version():
    subconfig = config['version']
    subconfig['third'] = str(int(subconfig['third']) + 1)
    version = [subconfig['first'], subconfig['second'], subconfig['third']]
    version = '.'.join(version)
    with open('config.ini', mode='w', encoding='utf-8') as file:
        config.write(file)
    return version


def get_readme():
    with open('README.md', mode='r', encoding='utf-8') as file:
        return file.read()


setup(
    name='pykindle',
    version=get_version(),
    author='Haoyu Pan',
    description="PyKindle: Generate mobi files in a programming way",
    long_description=get_readme(),
    author_email='haoyupan@aliyun.com',
    install_requires=['jinja2', 'markdown'],
    url="https://github.com/panhaoyu/pykindle",
    packages=find_packages(),
    license='GPL',
    classifiers=[
        'Intended Audience :: Developers',
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    project_urls={
        'Documentation': 'https://github.com/panhaoyu/pykindle',
        'Source': 'https://github.com/panhaoyu/pykindle',
    },
)
